/*
 * OnePlayerLevelSelect.cpp
 *
 *  Created on: 10 Dec 2009
 *      Author: math
 */

#include "OnePlayerLevelSelect.h"
#include "OnePlayerGameScreen.h"

OnePlayerLevelSelect::OnePlayerLevelSelect(MenuItem* newParent) : MenuItem(newParent) {
	if(plevelSelect == NULL) plevelSelect = this;
	gameScreenObject = new OnePlayerGameScreen(newParent);

	nextBox.x = 600;
	nextBox.w = 30;
	nextBox.y = 50;
	nextBox.h = 50;

	previousBox.x = 20;
	previousBox.w = 30;
	previousBox.y = 50;
	previousBox.h = 50;

	startBox.x = 120;
	startBox.w = 400;
	startBox.y = 390-FontHeight-2;
	startBox.h = 40;

	showbackbutton = true;
}

OnePlayerLevelSelect::~OnePlayerLevelSelect()  {
	printf("Desctructor for OnePlayerLevelSelect\n");
	delete(gameScreenObject);
}

void OnePlayerLevelSelect::SetLevelInformation(string new_title, int new_numberOfLemmings, int new_precentToBeSaved, int new_releaseRate, int new_time, string new_rating)
{
	levelNumber = levelLoader->currentLevel+1;
	title = new_title;
	numberOfLemmings = new_numberOfLemmings;
	precentToBeSaved = new_precentToBeSaved;
	releaseRate = new_releaseRate;
	time = new_time;
}

void OnePlayerLevelSelect::DrawSelf(SDL_Surface* screen){
	std::stringstream stream;
	stream.clear();

	//clear screen
	if(forceRedraw){
		forceRedraw = false;
		SDL_FillRect(screen, &screen->clip_rect, SDL_MapRGB( screen->format, 0x00, 0x00, 0x00 ) );
		TileBackground(screen);
		DrawLevelPreview(screen);

		int xPos = 150;
		int yPos = 180;
		int h = 32;
		stream << "Level " << levelLoader->currentLevel+1;
		fontRenderer->RenderString(screen, stream.str(), 1, yPos-32, RED, ABSOLUTE);

		stream.str("");
		stream << game->currentLevel->GetLevelName();
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos-32, RED, ABSOLUTE);

		stream.str("");
		stream << "Number of Lemmings " << numberOfLemmings;
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos, BLUE, ABSOLUTE);

		stream.str("");
		stream << precentToBeSaved << "% To Be Saved";
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos+=h, GREEN, ABSOLUTE);

		stream.str("");
		stream << "Release Rate " << releaseRate;
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos+=h, BROWN, ABSOLUTE);

		stream.str("");
		stream << "Time  " << time << " Minutes";
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos+=h, LIGHT_BLUE, ABSOLUTE);

		stream.str("");
		stream << "Rating  " << levelLoader->GetLevelSetName();
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos+=h, PURPLE, ABSOLUTE);


		stream.str("");
		stream << ">";
		fontRenderer->RenderString(screen, stream.str(), nextBox.x+5,  nextBox.y, GREEN, ABSOLUTE);

		if(levelLoader->currentLevel > 0){
		    stream.str("");
		    stream << "<";
		    fontRenderer->RenderString(screen, stream.str(), previousBox.x+5,  previousBox.y, GREEN, ABSOLUTE);
		}

		if(overnext)utility->DrawBox(screen,&nextBox, nextBox.x,nextBox.y);
		if(overprevious)utility->DrawBox(screen,&previousBox, previousBox.x,previousBox.y);
		if(overstart)utility->DrawBox(screen,&startBox, startBox.x,startBox.y+5);

		fontRenderer->RenderString(screen, "Click here to continue", 40, startBox.y, BLUE, CENTER);
	}
}

void OnePlayerLevelSelect::HandleKey(int k){
	if(k == SDLK_RETURN || k == SDLK_SPACE)
	{
		SetCurrentScreen(gameScreenObject);
	}
	else if(k == SDLK_RIGHT)
	{
		levelLoader->NextLevel();
		Init();
	}
	else if(k == SDLK_LEFT)
	{
		levelLoader->PreviousLevel();
		Init();
	}
	else if(k == SDLK_UP)
	{
		levelLoader->IncLevel(10);
		Init();
	}
	else if(k == SDLK_DOWN)
	{
		levelLoader->IncLevel(-10);
		Init();
	}
}

void OnePlayerLevelSelect::HandleMouse(int mx,int my,long lb,long rb){

	if(mx > nextBox.x && mx < nextBox.x+nextBox.w &&
							my > nextBox.y && my < nextBox.y+nextBox.h) {
			overprevious = false;
			overnext = true;
			overstart = false;
			overback = false;
			forceRedraw = true;
		}
		else if(levelLoader->currentLevel > 0 && mx > previousBox.x && mx < previousBox.x+previousBox.w &&
				my > previousBox.y && my < previousBox.y+previousBox.h) {
			overprevious = true;
			overnext = false;
			overstart = false;
			overback = false;
			forceRedraw = true;
		}else if(mx > startBox.x && mx < startBox.x+startBox.w &&
				my > startBox.y && my < startBox.y+startBox.h) {
			overprevious = false;
			overnext = false;
			overstart = true;
			overback = false;
			forceRedraw = true;
		}else if(mx > backBox.x && mx < backBox.x+backBox.w &&
				my > backBox.y && my < backBox.y+backBox.h) {
			overprevious = false;
			overnext = false;
			overstart = false;
			overback = true;
			forceRedraw = true;
		}
		else
		{
			if(overprevious || overnext || overstart || overback){
				overprevious = overnext = overstart = overback = false;
				forceRedraw = true;
			}
		}

	if(leftclick){

		//is it in the vicinity of 1-player?
		if(overprevious) {
			levelLoader->PreviousTwoPlayerLevel();
			Init();
			forceRedraw = true;
		}else if(overnext) {
			levelLoader->NextTwoPlayerLevel();
			Init();
			forceRedraw = true;
		}else if(overback) {
			SendEscape();
			forceRedraw = true;
		}
		else if(overstart){
			//ConnectToGame();
			SetCurrentScreen(gameScreenObject);
		}
	}

//	if(leftclick)
	//	SetCurrentScreen(gameScreenObject);
}

void OnePlayerLevelSelect::Init(){
	if(game != NULL){
		printf("OnPlayerLevelSelect: game->LoadLevel()\n");
		game->LoadLevel(levelLoader);
		printf("OnPlayerLevelSelect: game->Init()\n");
		game->Init(StatusPanel::SinglePlayer);
		printf("OnePlayerLevelSelect: game->SetLevelInformation()\n");

		SetLevelInformation(game->currentLevel->GetLevelName(),
							game->currentLevel->GetNumLemmings(),
							game->LemmingsNeededPercent(),
							game->currentLevel->GetReleaseRate(),
							game->currentLevel->GetTimeInMinutes(),
							game->currentLevel->GetRatingDescription());
	}
	MenuItem::Init();
}


