/*
 * ServerScreen.cpp
 *
 *  Created on: 18 Dec 2009
 *      Author: mat
 */

#include "ServerLevelSelect.h"

ServerLevelSelect::ServerLevelSelect(MenuItem* daddy) : MenuItem(daddy){
	serverGameScreen = new ServerGameScreen(this);
	connected = false;

	nextBox.x = 600;
	nextBox.w = 30;
	nextBox.y = 50;
	nextBox.h = 50;

	previousBox.x = 20;
	previousBox.w = 30;
	previousBox.y = 50;
	previousBox.h = 50;

	startBox.x = 160;
	startBox.w = 320;
	startBox.y = 390-FontHeight-2;
	startBox.h = 40;

	showbackbutton = true;
}

ServerLevelSelect::~ServerLevelSelect() {
	printf("Desctructor for ServerLevelSelect\n");
	delete(serverGameScreen);
}

void ServerLevelSelect::DrawSelf(SDL_Surface* screen){
	std::stringstream stream;
	stream.clear();
	if(forceRedraw){
		forceRedraw = false;
		//clear screen
		SDL_FillRect(screen, &screen->clip_rect, SDL_MapRGB( screen->format, 0x00, 0x00, 0x00 ) );

		TileBackground(screen);

		DrawLevelPreview(screen);

		int xPos = 180;
		int yPos = 180;
		int h = 32;
		stream << "Level " << levelLoader->currentLevel+1;
		fontRenderer->RenderString(screen, stream.str(), 1, yPos-32, RED, ABSOLUTE);

		stream.str("");
		stream << game->currentLevel->GetLevelName();
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos-32, RED, ABSOLUTE);

		stream.str("");
		stream << "(you) Blue  Player has " << game->blueGameMaxLems << " Lemmings" ;
		fontRenderer->RenderString(screen, stream.str(), 10, yPos+=h, BLUE, ABSOLUTE);

		stream.str("");
		stream << "      Green Player has " << game->greenGameMaxLems << " Lemmings " ;
		fontRenderer->RenderString(screen, stream.str(), 10, yPos+=h, GREEN, ABSOLUTE);

		stream.str("");
		stream << "Release Rate " << game->currentLevel->GetReleaseRate();
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos+=h, BROWN, CENTER);

		if(!sleeping)
		{
			stream.str("");
			stream << ">";
			fontRenderer->RenderString(screen, stream.str(), nextBox.x+5,  nextBox.y, GREEN, ABSOLUTE);

			if(levelLoader->currentLevel > 0){
				stream.str("");
				stream << "<";
				fontRenderer->RenderString(screen, stream.str(), previousBox.x+5,  previousBox.y, GREEN, ABSOLUTE);
			}

			if(overnext)utility->DrawBox(screen,&nextBox, nextBox.x,nextBox.y);
			if(overprevious)utility->DrawBox(screen,&previousBox, previousBox.x,previousBox.y);
			if(overstart)utility->DrawBox(screen,&startBox, startBox.x,startBox.y+5);
		}
	}

	if(!waitingForConnection){
		//fontRenderer->RenderString(screen, "Press left button to start", 40, 400-FontHeight-2, BLUE, CENTER);
		fontRenderer->RenderString(screen, "Click Here to start", 40, startBox.y, BLUE, CENTER);
	}
	else
	{
		fontRenderer->RenderString(screen, "Waiting for client to connect...", 40, 400-FontHeight-2, BLUE, CENTER);
		if(!connected){
			connected = game->WaitForClientToConnect();
			if(connected){
				sleeping = false;
				printf("server connected, yay set to game screen\n");
				//if now connected send the level to the client, before changin screen
				game->SetClientLevel(levelLoader->currentLevel);
				SetCurrentScreen(serverGameScreen);
			}
		}
	}
}

MenuItem* ServerLevelSelect::SendEscape() {
	game->CloseConnection();
	return MenuItem::SendEscape();
}

void ServerLevelSelect::HandleKey(int k){
	if(k == SDLK_RETURN){
		if(!waitingForConnection){
			ConnectToGame();
		}
	}
	if(!connected){
		if(k == SDLK_RIGHT )
		{
			forceRedraw=true;
			levelLoader->NextTwoPlayerLevel();
			Init();
		}
		else if(k == SDLK_LEFT)
		{
			forceRedraw=true;
			levelLoader->PreviousTwoPlayerLevel();
			Init();
		}
	}
}

void ServerLevelSelect::HandleMouse(int mx,int my,long lb,long rb){
	if(mx > nextBox.x && mx < nextBox.x+nextBox.w &&
						my > nextBox.y && my < nextBox.y+nextBox.h) {
		overprevious = false;
		overnext = true;
		overstart = false;
		overback = false;
		forceRedraw = true;
	}
	else if(levelLoader->currentLevel > 0 && mx > previousBox.x && mx < previousBox.x+previousBox.w &&
			my > previousBox.y && my < previousBox.y+previousBox.h) {
		overprevious = true;
		overnext = false;
		overstart = false;
		overback = false;
		forceRedraw = true;
	}else if(mx > startBox.x && mx < startBox.x+startBox.w &&
			my > startBox.y && my < startBox.y+startBox.h) {
		overprevious = false;
		overnext = false;
		overstart = true;
		overback = false;
		forceRedraw = true;
	}else if(mx > backBox.x && mx < backBox.x+backBox.w &&
			my > backBox.y && my < backBox.y+backBox.h) {
		overprevious = false;
		overnext = false;
		overstart = false;
		overback = true;
		forceRedraw = true;
	}
	else
	{
		if(overprevious || overnext || overstart ){
			overprevious = overnext = overstart = false;
			forceRedraw = true;
		}
	}


	if(leftclick){

		//is it in the vicinity of 1-player?
		if(overprevious && !sleeping) {
			levelLoader->PreviousTwoPlayerLevel();
			Init();
			forceRedraw = true;
		}else if(overnext && !sleeping) {
			levelLoader->NextTwoPlayerLevel();
			Init();
			forceRedraw = true;
		}else if(overback && !sleeping) {
			SendEscape();
			forceRedraw = true;
		}
		else if(overstart){
			ConnectToGame();
		}
	}
}

void ServerLevelSelect::ConnectToGame(){
	if(!waitingForConnection){
		waitingForConnection = true;
		sleeping = true;
		forceRedraw = true;
		game->ConnectToGame(StatusPanel::Server,"");
	}
}

void ServerLevelSelect::Reset(){
	sleeping = false;
	MenuItem::Reset();
}

void ServerLevelSelect::Init(){
	printf("loading two player level\n");
	game->Load2PlayerLevel(levelLoader);
	printf("game->init\n");
	game->Init(StatusPanel::Server);
	waitingForConnection = false;
	connected = false;
	printf("done a ConnectToGame as server\n");
	printf("Initiated serverLevelSelect and Game object for server.\n");
	MenuItem::Init();
}

